<?php

require_once 'Horde/MIME/Contents.php';
require_once MIMP_BASE . '/config/mime_drivers.php';

/**
 * The MIMP_Contents:: class extends the MIME_Contents:: class and contains
 * all functions related to handling the content and output of mail messages
 * in MIMP.
 *
 * $Horde: mimp/lib/MIME/Contents.php,v 1.35 2005/01/03 14:35:17 jan Exp $
 *
 * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
 * @version $Revision: 1.35 $
 * @since   MIMP 0.0.1
 * @package MIMP
 */
class MIMP_Contents extends MIME_Contents {

    /**
     * The text of the body of the message.
     *
     * @var string $_body
     */
    var $_body = '';

    /**
     * The text of various MIME body parts.
     *
     * @var array $_bodypart
     */
    var $_bodypart = array();

    /**
     * The IMAP index of the message.
     *
     * @var integer $_index
     */
    var $_index;

    /**
     * Constructor
     *
     * @access public
     *
     * @param integer $index              The IMAP message index.
     */
    function MIMP_Contents($index)
    {
        $this->_index = $index;

        /* Get the MIME_Message object for the given index. */
        require_once 'Horde/MIME/Structure.php';
        $ob = &MIME_Structure::parse(@imap_fetchstructure($_SESSION['mimp']['stream'], $index, FT_UID));
        $ob->setInformation('index', $index);

        parent::MIME_Contents($ob);
    }

    /**
     * Returns the entire body of the message.
     *
     * @access public
     *
     * @return string  The text of the body of the message.
     */
    function getBody()
    {
        if (empty($this->_body)) {
            $this->_body = @imap_body($_SESSION['mimp']['stream'], $this->_index, FT_UID);
        }

        return $this->_body;
    }

    /**
     * Gets the raw text for one section of the message.
     *
     * @access public
     *
     * @param integer $id  The ID of the MIME_Part.
     *
     * @return string  The text of the part.
     */
    function getBodyPart($id)
    {
        $contents = '';

        if (!isset($this->_bodypart[$id])) {
            $this->_bodypart[$id] = @imap_fetchbody($_SESSION['mimp']['stream'], $this->_index, $id, FT_UID);
        }

        return ($contents . $this->_bodypart[$id]);
    }

    /**
     * Return the attachment list, in Horde_Mobile:: objects for
     * header listing.
     *
     * @access public
     *
     * @param object Horde_Mobile_block $hb  The Horde_Mobile_block object to add to.
     */
    function getAttachments(&$hb)
    {
        $msg = '';

        foreach ($this->_atc as $key => $val) {
            $hb->add(new Horde_Mobile_text(_("Attachment") . ': ', array('b')));
            $s = &$this->_atc[$key];
            $t = &$hb->add(new Horde_Mobile_text(sprintf('%s (%s, %s)', $s[2], $s[3], $s[4]) . "\n"));
            $t->set('linebreaks', true);
        }
    }

    /**
     * Return all viewable message parts (plain text, for
     * Horde_Mobile:: to deal with).
     *
     * @access public
     *
     * @return string  The full message.
     */
    function getMessage()
    {
        $msg = '';
        $partDisplayed = false;

        foreach ($this->_parts as $key => $value) {
            if (!empty($value)) {
                if ($msg) {
                    $msg .= "\n\n";
                }
                $msg .= $value;
                $partDisplayed = true;
            }
        }

        if (!$partDisplayed) {
            $msg .= _("There are no parts that can be displayed inline.");
        }

        return $msg;
    }

    /**
     * Returns an array summarizing a part of a MIME message.
     *
     * @access public
     *
     * @param object MIME_Part &$mime_part  The MIME_Part to summarize.
     * @param optional boolean $guess       Is this a temporary guessed-type
     *                                      part?
     *
     * @return array  See MIME_Contents::partSummary().
     *                Ignores guessed parts.
     */
    function partSummary(&$mime_part, $guess = false)
    {
        if ($guess) {
            return array();
        } else {
            return parent::partSummary($mime_part);
        }
    }

    /**
     * Returns the full message text for a given message/mailbox.
     *
     * @access public
     *
     * @param optional integer $index  The index/mailbox of the message. If
     *                                 empty, will use the current message
     *                                 index.
     *
     * @return string  The full message text.
     */
    function fullMessageText($index = null)
    {
        if (is_null($index)) {
            $index = $this->_index;
        } elseif ($index != $this->_index) {
            $this->_body = '';
            $this->_index = $index;
        }

        require_once MIMP_BASE . '/lib/MIME/Headers.php';
        $mimp_headers = &new MIMP_Headers($index);
        return $mimp_headers->getHeaderText() . $this->getBody();
    }

    /**
     * Rebuild the MIME_Part structure of a message from IMAP data.
     *
     * @access public
     *
     * @param optional boolean $decode  Decode the body parts?
     *
     * @return object MIME_Message  A MIME_Message object with all of the body
     *                              text stored in the individual MIME_Parts.
     */
    function rebuildMessage($decode = false)
    {
        $part = $this->_message->getBasePart();
        $this->_rebuildMessage($part, $decode);

        return $this->_message;
    }

    /**
     * Recursive function used to rebuild the MIME_Part structure of a
     * message.
     *
     * @access private
     *
     * @param object MIME_Part &$part  A MIME_Part object.
     * @param boolean $decode          Decode the body parts?
     */
    function _rebuildMessage(&$part, $decode)
    {
        $id = $part->getMIMEId();

        if ($part->getPrimaryType() == 'multipart') {
            /* Recursively process any subparts. */
            foreach ($part->getParts() as $mime) {
                $this->_rebuildMessage($mime, $decode);
            }
        } elseif ($part->getType() == 'message/rfc822') {
            $part->setContents($this->getBodyPart(substr($id, 0, strrpos($id, '.'))));
            $this->_message->alterPart($id, $part);
        } else {
            if ($decode) {
                $this->_message->alterPart($id, $this->getDecodedMIMEPart($id));
            } else {
                $mime_part = &$this->getMIMEPart($id);
                $mime_part->setContents($this->getBodyPart($id));
                $this->_message->alterPart($id, $mime_part);
            }
        }
    }

    /**
     * Render a MIME Part.
     *
     * @param object MIME_Part &$mime_part  A MIME_Part object.
     *
     * @return string  The rendered data.
     */
    function renderMIMEPart(&$mime_part)
    {
        if (!$mime_part->getContents()) {
            $mime_part->setContents($this->getBodyPart($mime_part->getMIMEId()));
        }
        return parent::renderMIMEPart($mime_part);
    }

}
